// -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Jason Smith <jason.smith@canonical.com>
 */

#include "AbstractLauncherIcon.h"

namespace unity {
namespace launcher {
  nux::Property<unsigned> AbstractLauncherIcon::icon_size(48);
  nux::Property<bool> AbstractLauncherIcon::scroll_inactive_icons(true);
  nux::Property<bool> AbstractLauncherIcon::minimize_window_on_click(false);

  // needed for ungodly stupid reasons
  NUX_IMPLEMENT_OBJECT_TYPE(AbstractLauncherIcon);
}
}
